package application;

import io.XmlConverter;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Timer;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;

import extern.BufferedImageDrawer;
import extern.MorphingCandS;
import graphic2D.Triangle;
import gui.GUI;
import gui.ImagePanel;

public class MainApp extends App {
	BufferedImage leftimage;

	BufferedImage rightimage;

	MorphingCandS mcs;

	BufferedImage bi;

	BufferedImageDrawer bid;

	// Vector<Point2D> points;

	public MainApp() {
		leftimage = null;
		rightimage = null;
		bi = null;

	}

	private BufferedImage openImage(String filename) {
		BufferedImage bimg;
		try {
			Image img = new ImageIcon(filename).getImage();
			if ((img != null) && (img.getWidth(null) >= 0)
					&& (img.getHeight(null) >= 0)) {
				bimg = new BufferedImage(img.getWidth(null), img
						.getHeight(null), BufferedImage.TYPE_INT_RGB);
				Graphics2D g2d = bimg.createGraphics();
				g2d.drawImage(img, 0, 0, null);
			} else {
				bimg = null;
			}

		} catch (Exception e) {
			bimg = null;
			e.printStackTrace();
		}
		return bimg;
	}

	@Override
	public boolean openLeftImage(String filename) {
		BufferedImage bi = openImage(filename);
		if (bi != null) {
			leftimage = bi;
			if (gui != null) {
				gui.setLeftImage(bi);
			}
		}
		return bi != null;
	}

	@Override
	public boolean openRightImage(String filename) {
		BufferedImage bi = openImage(filename);
		if (bi != null) {
			rightimage = bi;
			if (gui != null) {
				gui.setRightImage(bi);
			}
		}
		return bi != null;
	}

	private boolean saveImage(RenderedImage img, String filename) {
		boolean saved;
		try {
			ImageIO.write(img, "png", new File(filename));
			saved = true;
		} catch (IOException e) {
			// e.printStackTrace();
			saved = false;
		}
		return saved;
	}

	/*
	 * @Override public boolean saveLeftImage(String filename) { return
	 * saveImage(leftimage, filename); }
	 * 
	 * @Override public boolean saveRightImage(String filename) { return
	 * saveImage(rightimage, filename); }
	 */
	@Override
	public int exit() {
		System.out.println("EXIT");
		return App.APP_RET_OK;
	}

	@Override
	public void setGui(GUI gui) {
		this.gui = gui;
	}

	@Override
	public GUI getGui() {
		return this.gui;
	}

	@Override
	public void execute(Vector<Point2D> srcPoints,
			Vector<Triangle> srcTriangles, Image srcImg,
			Vector<Point2D> dstPoints, Vector<Triangle> dstTriangles,
			Image dstImg) {
		// Breite des Fensters
		// int width = 200;
		// Hoehe des Fensters
		// int height = 250;

		// Gibt an (in Millisekunden), nach wie langer Zeit das Bild
		// aktualisiert
		// werden soll.
		int delay = 150;
		if (bi != null)
			bid.dispose();
		// Das BufferedImage, das im Fenster jeweils gezeichnet wird.
		bi = new BufferedImage(srcImg.getWidth(null), srcImg.getHeight(null),
				BufferedImage.TYPE_INT_RGB);

		// Das Fenster, in dem alles angezeigt wird.
		bid = new BufferedImageDrawer(bi, srcImg.getWidth(null), srcImg
				.getHeight(null));
		bid.setTitle("Umwandlung von Form und Farbe");

		this.mcs = new MorphingCandS(getGui(), bid, srcPoints, srcTriangles,
				srcImg, dstPoints, dstTriangles, dstImg);
		Timer t = new Timer();
		t.scheduleAtFixedRate(mcs, 0, delay);

	}

	@Override
	public boolean openProject(String filename) {
		boolean success = false;
		if (gui != null) {
			Document doc;
			File f=null;
			try {
				f=new File(filename);
				doc = XmlConverter.getJDOMDocument(f);
			} catch (JDOMException e) {
				doc = null;
				e.printStackTrace();
			} catch (IOException e) {
				doc = null;
				e.printStackTrace();
			}
			if (doc != null) {
				String file_separator = System.getProperty("file.separator");
				String path=f.getPath().substring(0,f.getPath().lastIndexOf(file_separator))+file_separator;
				readImageConf(path, doc, "leftimage", gui.getLeftImagePanel());
				readImageConf(path, doc, "rightimage", gui.getRightImagePanel());
				boolean b = gui.getLeftImagePanel().completeToCommit();
				gui.enableCommitBtn(b);
				gui.enableExecuteBtn(b&&(gui.getRightImagePanel().getTriangles().size()!=0));
			}
		}
		return success;
	}

	@SuppressWarnings("unchecked")
	private void readImageConf(String path, Document doc, String strImage,
			ImagePanel imagePanel) {
		Element imgEl = doc.getRootElement().getChild("project").getChild(
				strImage);
		if (imgEl != null) {
			String fileName = imgEl.getAttributeValue("filename");
			if (fileName != null) {
				System.out.println("f="+path+fileName);
				BufferedImage bimg = openImage(path+fileName);
				if (bimg != null) {
					imagePanel.setImage(bimg, false);
					imagePanel.clearAll();
					Element pointsEl = imgEl.getChild("points");
					if (pointsEl != null) {
						Vector<Point2D> points = new Vector<Point2D>();
						Vector<Triangle> triangles = new Vector<Triangle>();
						List<Element> lEl = (List<Element>) pointsEl
								.getChildren("point");
						Iterator<Element> itEl = lEl.iterator();
						while (itEl.hasNext()) {
							Element el = itEl.next();
							Point2D p = new Point2D.Double(new Double(el
									.getAttributeValue("x")), new Double(el
									.getAttributeValue("y")));
							points.add(p);
						}
						imagePanel.setPoints(points);
						Element trianglesEl = imgEl.getChild("triangles");
						if (trianglesEl != null) {
							lEl = (List<Element>) trianglesEl
									.getChildren("triangle");
							itEl = lEl.iterator();
							while (itEl.hasNext()) {
								Element el = itEl.next();
								int refP1 = new Integer(el
										.getAttributeValue("p1"));
								int refP2 = new Integer(el
										.getAttributeValue("p2"));
								int refP3 = new Integer(el
										.getAttributeValue("p3"));
								Triangle t = new Triangle(points.get(refP1),
										points.get(refP2), points.get(refP3));
								triangles.add(t);
							}
							imagePanel.setTriangles(points, triangles);
						}
					}
				}
			}
		}
	}

	@Override
	public boolean saveProject(String filename) {
		boolean success = false;
		if (gui != null) {
			Document doc;
			Element currentElem;
			Element newElem;
			String projectName = getProjectName(filename);
			doc = new Document();// XmlConverter.getJDOMDocument(new
			// File(filename));
			newElem = new Element("Root");
			doc.setRootElement(newElem);
			currentElem = newElem;
			newElem = new Element("project");
			newElem.setAttribute(new Attribute("name", projectName));
			currentElem.addContent(newElem);
			currentElem = newElem;
			success = saveImage(currentElem, "leftimage", gui
					.getLeftImagePanel(), filename);
			if (success)
				success = saveImage(currentElem, "rightimage", gui
						.getRightImagePanel(), filename);
			if (success) {
				File f = new File(filename);
				try {
					BufferedWriter bw = new BufferedWriter(new FileWriter(f));
					bw.write(XmlConverter.getXMLString(doc));
					bw.flush();
				} catch (IOException e) {
					success = false;
					e.printStackTrace();
				}
			}
		}
		return success;
	}

	private String getPath(String filename) {
		String file_separator = System.getProperty("file.separator");
		String path = filename.substring(0, filename
				.lastIndexOf(file_separator));
		if (path.length() == 0)
			path = ".";
		return path + file_separator;
	}

	private boolean saveImage(Element currentElem, String tag,
			ImagePanel imagePanel, String filename) {
		boolean success = true;
		String file_separator = System.getProperty("file.separator");
		if ((imagePanel != null) && imagePanel.getImage() != null) {
			Element newElem = new Element(tag);
			(new File(getPath(filename) + getProjectName(filename)
					+ file_separator + "images")).mkdirs();
			if (success) {
				String salt = Double.toHexString(Math.random()).trim()
						.replaceAll("^[0-9A-Fa-f]", "");

				String imageFileName1 = getProjectName(filename)
						+ file_separator + "images" + file_separator
						+ getProjectName(filename) + salt + ".png";
				String imageFileName = getPath(filename) + imageFileName1;
				Image img = imagePanel.getImage();
				BufferedImage bimg;
				if (!(img instanceof BufferedImage)) {
					bimg = new BufferedImage(img.getWidth(null), img
							.getHeight(null), BufferedImage.TYPE_INT_RGB);
					Graphics2D g2d = bimg.createGraphics();
					g2d.drawImage(img, 0, 0, null);
				} else {
					bimg = (BufferedImage) img;
				}
				success = saveImage(bimg, imageFileName);
				if (success) {
					newElem.setAttribute(new Attribute("filename",
							imageFileName1));
					currentElem.addContent(newElem);
					currentElem = newElem;
					success = assembleSetData(currentElem, imagePanel);
				}
			} else {

			}
		}
		return success;
	}

	private boolean assembleSetData(Element currentElem, ImagePanel imagePanel) {
		boolean success = true;
		Vector<Point2D> points = imagePanel.getPoints();
		if ((points != null) && (points.size() > 0)) {
			Element elPoints = new Element("points");
			currentElem.addContent(elPoints);
			for (int i = 0; i < points.size(); i++) {
				Element p = new Element("point");
				p.setAttribute(new Attribute("x", Double.toString(points.get(i)
						.getX())));
				p.setAttribute(new Attribute("y", Double.toString(points.get(i)
						.getY())));
				elPoints.addContent(p);
			}
			Vector<Triangle> triangles = imagePanel.getTriangles();
			if ((triangles != null) && (triangles.size() > 0)) {
				Element elTriangles = new Element("triangles");
				currentElem.addContent(elTriangles);
				for (int i = 0; i < triangles.size(); i++) {
					Element elT = new Element("triangle");
					Triangle t = triangles.get(i);
					Vector<Point2D> coners = t.getConers();
					for (int c = 0; c < coners.size(); c++) {
						int idx = points.indexOf(coners.get(c));
						success = success && (idx != -1);
						if (!success)
							System.out.println("point not found!");
						elT.setAttribute(new Attribute("p"
								+ Integer.toString(c + 1), Integer
								.toString(idx)));
					}
					elTriangles.addContent(elT);
				}
			}
		}
		return success;
	}

	private String getProjectName(String filename) {
		String file_separator = System.getProperty("file.separator");
		int start_idx = filename.lastIndexOf(file_separator);
		int end_idx = filename.lastIndexOf(".");
		if (start_idx < 0)
			start_idx = 0;
		else
			start_idx++;
		if (end_idx < start_idx)
			end_idx = filename.length();
		return filename.substring(start_idx, end_idx);
	}

}
