package graphic2D;

import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Vector;

public class Triangle {
	private Point2D points[];

	public static final double eps = 0.0001;

	public Triangle(Point2D p0, Point2D p1, Point2D p2) {
		this.points = new Point2D[3];
		this.points[0] = p0;
		this.points[1] = p1;
		this.points[2] = p2;
	}

	public boolean isTriangle() {
		return points[0].distance(points[1]) + points[0].distance(points[2])
				- points[1].distance(points[2]) > eps;
	}

	public boolean isPointIn(Point2D p) {
		boolean isIn = true;
		for (int i = 0; isIn && (i < 3); i++) {
			double alphaF = getAlpha(points[(i + 1) % 3], points[(i + 2) % 3],
					p);
			Point2D projectionsFremdPunkt = getP(points[(i + 1) % 3],
					points[(i + 2) % 3], alphaF);
			double alphaE = getAlpha(points[(i + 1) % 3], points[(i + 2) % 3],
					points[i]);
			Point2D projectionEigenesEckpunktes = getP(points[(i + 1) % 3],
					points[(i + 2) % 3], alphaE);
			double sig1 = projectionsFremdPunkt.getX() - p.getX();
			if (Math.abs(sig1) < eps)
				sig1 = 0.0;
			double sig2 = projectionEigenesEckpunktes.getX() - points[i].getX();
			if (Math.abs(sig2) < eps)
				sig2 = 0.0;
			// double sig3;
			// double sig4;

			isIn = isIn
					&& (((Math.abs(sig1)) < eps) || (Math.abs(sig2) < eps) || (Math
							.signum(sig1) == Math.signum(sig2)))

					&& ((Math.abs(projectionsFremdPunkt.getY() - p.getY()) < eps)
							|| (Math.abs(projectionEigenesEckpunktes.getY()
									- points[i].getY()) < eps) || (Math
							.signum(projectionsFremdPunkt.getY() - p.getY()) == Math
							.signum(projectionEigenesEckpunktes.getY()
									- points[i].getY())));
		}
		return isIn;
	}

	private Point2D getP(Point2D p1, Point2D p2, double alpha) {
		double dx = p2.getX() - p1.getX();
		double dy = p2.getY() - p1.getY();
		return new Point2D.Double(p1.getX() + alpha * dx, p1.getY() + alpha
				* dy);
	}

	public static double getAlpha(Point2D p0, Point2D p1, Point2D p2) {
		return ((p0.getX() - p1.getX()) * (p0.getX() - p2.getX()) + (p0.getY() - p1
				.getY())
				* (p0.getY() - p2.getY()))
				/ ((p0.getX() - p1.getX()) * (p0.getX() - p1.getX()) + (p0
						.getY() - p1.getY())
						* (p0.getY() - p1.getY()));

	}

	public boolean hasAConer(Point2D p) {
		int i = 0;
		while ((i < points.length) && (!points[i].equals(p)))
			i++;
		return i < points.length;
	}

	public boolean equals(Object o) {
		int n = 0;
		if (o instanceof Triangle) {
			Triangle t = (Triangle) o;
			for (int i = 0; i < points.length; i++) {
				for (int j = 0; j < t.points.length; j++) {
					if (points[i].equals(t.points[j]))
						n++;
				}
			}
		}
		return n == 3;
	}

	public Vector<Point2D> getConers() {
		Vector<Point2D> v = new Vector<Point2D>();
		for (int i = 0; i < 3; i++)
			v.add(points[i]);
		return v;
	}

	public Vector<Line2D> getEdges() {
		Vector<Line2D> edges = new Vector<Line2D>();
		for (int i = 0; i < 3; i++) {
			Line2D l = new Line2D.Double(points[i], points[(i + 1) % 3]);
			edges.addElement(l);
		}
		return edges;
	}

	public Object clone() {
		return new Triangle((Point2D) points[0].clone(), (Point2D) points[1]
				.clone(), (Point2D) points[2].clone());
	}
	
	public String toString(){
		return "Triangle "+points[0].toString()+","+points[1].toString()+","+points[2].toString();
	}
}
