/**
 * 
 */
package gui;

import gui.listener.AppMouseMotionAdapter;
import gui.listener.AppWindowAdapter;
import gui.listener.MenuFileActionListener;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.MouseMotionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;

import application.App;

/**
 * @author pavel
 * 
 */
public class GUI extends JFrame {
	/**
	 * 
	 */
	private static final long serialVersionUID = -7719030036308427218L;

	protected App app;

	protected static final int LEFT = 0;

	protected static final int RIGHT = 1;

	protected ImagePanel[] ps = { new ImagePanel(), new ImagePanel() };

	protected JPanel statebar;
	
	protected JButton btnCommit = new JButton("Dreiecke bernehmen");
	protected JButton btnExecute = new JButton("DO IT!");
	
	protected boolean triangleMode = true;// the init doClick() on btnSetPoint
											// button, chages it at beginning to
											// false

	public final static String MENU_FILE_OPEN_LEFT_IMAGE = "Open Left Image";

	public final static String MENU_FILE_OPEN_RIGHT_IMAGE = "Open Right Image";

	public final static String MENU_FILE_OPEN_PROJECT = "Open Project";

	public final static String MENU_FILE_SAVE_PROJECT = "Save Project";

	public final static String MENU_FILE_EXIT = "Exit";
	
	public final static String MENU_HELP_HELP = "Help";
	public final static String MENU_HELP_ABOUT = "About";

	private final static String SCSize = "32x32";

	public final static String SC_OPEN_LEFT_IMAGE = "images/" + GUI.SCSize
			+ "/open_l.png";

	public final static String SC_OPEN_RIGHT_IMAGE = "images/" + GUI.SCSize
			+ "/open_r.png";

	public final static String SC_OPEN_PROJECT = "images/" + GUI.SCSize
			+ "/open_p.png";

	public final static String SC_SAVE_PROJECT = "images/" + GUI.SCSize
			+ "/save_p.png";

	public GUI(App app, String title) {
		super(title);
		this.app = app;
		app.setGui(this);
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		this.setSize((int) Math.round(d.getWidth() * 2 / 3), (int) Math.round(d
				.getHeight() * 2 / 3));
		this.setLocation((d.width - this.getSize().width) / 2, (d.height - this
				.getSize().height) / 2);
		buildMenu();
		btnCommit.setEnabled(false);
		btnExecute.setEnabled(false);
		this.setVisible(true);
	}

	private void buildMenu() {
		JMenuBar menubar = new JMenuBar();
		JMenu filemenu = new JMenu("File");
		JMenuItem openLeftItem = new JMenuItem(GUI.MENU_FILE_OPEN_LEFT_IMAGE);
		JMenuItem openRightItem = new JMenuItem(GUI.MENU_FILE_OPEN_RIGHT_IMAGE);
		JMenuItem openProjectItem = new JMenuItem(GUI.MENU_FILE_OPEN_PROJECT);
		JMenuItem saveProjectItem = new JMenuItem(GUI.MENU_FILE_SAVE_PROJECT);
		JMenuItem exititem = new JMenuItem(GUI.MENU_FILE_EXIT);
		JMenu helpmenu = new JMenu("Help");
		JMenuItem helpitem = new JMenuItem(GUI.MENU_HELP_HELP);
		JMenuItem aboutitem = new JMenuItem(GUI.MENU_HELP_ABOUT);
		
		
		JToolBar toolbar = new JToolBar();

		MenuFileActionListener aal = new MenuFileActionListener(this.app);
		AppWindowAdapter awa = new AppWindowAdapter(this.app);
		AppMouseMotionAdapter mma = new AppMouseMotionAdapter(this.app);

		openLeftItem.addActionListener(aal);
		openRightItem.addActionListener(aal);
		openProjectItem.addActionListener(aal);
		saveProjectItem.addActionListener(aal);
		exititem.addActionListener(aal);
		
		helpitem.addActionListener(aal);
		aboutitem.addActionListener(aal);
		
		this.addWindowListener(awa);

		filemenu.add(openLeftItem);
		filemenu.add(openRightItem);
		filemenu.addSeparator();
		filemenu.add(openProjectItem);
		filemenu.add(saveProjectItem);
		filemenu.addSeparator();
		filemenu.add(exititem);
		
		helpmenu.add(helpitem);
		helpmenu.addSeparator();
		helpmenu.add(aboutitem);
		
		menubar.add(filemenu);
		menubar.add(helpmenu);
		this.setJMenuBar(menubar);

		JButton btnOL = new JButton(new ImageIcon(SC_OPEN_LEFT_IMAGE));
		btnOL.setToolTipText(GUI.MENU_FILE_OPEN_LEFT_IMAGE);
		JButton btnOR = new JButton(new ImageIcon(SC_OPEN_RIGHT_IMAGE));
		btnOR.setToolTipText(GUI.MENU_FILE_OPEN_RIGHT_IMAGE);
		JButton btnOP = new JButton(new ImageIcon(SC_OPEN_PROJECT));
		btnOP.setToolTipText(GUI.MENU_FILE_OPEN_PROJECT);
		JButton btnSP = new JButton(new ImageIcon(SC_SAVE_PROJECT));
		btnSP.setToolTipText(GUI.MENU_FILE_SAVE_PROJECT);

		
		
		Vector<JButton> v = new Vector<JButton>();
		MyButton btnSetTriangle = new MyButton("Triangle");
		MyButton btnSetPoint = new MyButton("Point");
		v.add(btnSetPoint);
		btnSetTriangle.setOthers(v);
		v = new Vector<JButton>();
		v.add(btnSetTriangle);
		btnSetPoint.setOthers(v);
		v = null;

		btnOL.addActionListener(aal);
		btnOR.addActionListener(aal);
		btnOP.addActionListener(aal);
		btnSP.addActionListener(aal);
		btnSetPoint.addActionListener(aal);
		btnSetTriangle.addActionListener(aal);
		btnCommit.addActionListener(aal);
		btnExecute.addActionListener(aal);

		
		
		toolbar.add(btnOL);
		toolbar.add(btnOR);
		toolbar.addSeparator();
		toolbar.add(btnOP);
		toolbar.add(btnSP);
		toolbar.addSeparator();

		toolbar.add(btnSetPoint);
		btnSetPoint.doClick();
		toolbar.add(btnSetTriangle);
		toolbar.addSeparator();
		toolbar.add(btnCommit);
		toolbar.addSeparator();
		toolbar.add(btnExecute);
		
		MouseMotionListener mml[] = toolbar.getMouseMotionListeners();
		for (int i = 0; i < mml.length; i++) {
			toolbar.removeMouseMotionListener(mml[i]);
		}
		this.getContentPane().add(toolbar, BorderLayout.NORTH);
		JPanel p = new JPanel();
		JPanel[] n_ps = { new JPanel(), new JPanel() };

		this.getContentPane().add(p);
		p.setLayout(new GridLayout(1, 2));
		for (int i = 0; i < 2; i++) {
			n_ps[i].setBorder(new BevelBorder(BevelBorder.LOWERED));
			ps[i].setBorder(new BevelBorder(BevelBorder.RAISED));
			n_ps[i].setLayout(new GridLayout(1, 1));
			ps[i].setLayout(new GridLayout(1, 1));
			ps[i].addMouseMotionListener(mma);
			ps[i].addMouseListener(mma);
			// ps[i].setVisible(true);
			n_ps[i].add(ps[i]);
			p.add(n_ps[i]);
		}
		p.setVisible(true);
		// this.getContentPane().add(p);
		statebar = new JPanel(new FlowLayout(FlowLayout.LEFT));
		statebar.setBorder(BorderFactory.createEtchedBorder());
		this.getContentPane().add(statebar, BorderLayout.SOUTH);

	}

	public void setStateText(String text) {
		JLabel state = new JLabel(text, SwingConstants.LEFT);
		state.setFont(new Font("Arial", Font.CENTER_BASELINE, 10));
		statebar.removeAll();
		statebar.validate();
		getContentPane().validate();
		statebar.add(state);
		statebar.validate();
		getContentPane().validate();
	}

	public void setLeftImage(Image img) {
		ps[LEFT].setImage(img,true);
		getContentPane().validate();
	}

	public void setRightImage(Image img) {
		ps[RIGHT].setImage(img,false);
		getContentPane().validate();
		btnCommit.setEnabled(getLeftImagePanel().completeToCommit());
	}

	public ImagePanel getLeftImagePanel() {
		return ps[LEFT];
	}

	public ImagePanel getRightImagePanel() {
		return ps[RIGHT];
	}

	public boolean isTriangleMode() {
		return triangleMode;
	}

	public void toggleTriangleMode() {
		this.triangleMode = !this.triangleMode;
	}

	public boolean setTriangleMode(boolean triangleMode) {
		this.triangleMode = triangleMode;
		return this.triangleMode;
	}
	
	public void enableCommitBtn(boolean en){
		btnCommit.setEnabled(en);
	}

	public void enableExecuteBtn(boolean b) {
		btnExecute.setEnabled(b);
		
	}
	
}
