package gui;

import graphic2D.Triangle;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Polygon;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

public class ImagePanel extends JPanel {

	protected Vector<Point2D> points;

	protected Vector<Triangle> triangles;

	protected Vector<Point2D> currentTriangle;

	protected int radius;

	/**
	 * 
	 */
	private static final long serialVersionUID = 3472558412618524409L;

	protected Image img;

	AffineTransform scaleTransform;

	AffineTransform inverseScaleTransform;

	private int flashingTriangle = -1;

	public ImagePanel() {
		this((Image) null);
	}

	public ImagePanel(Image img) {
		super();
		this.img = img;

		// alle gestzte punkte
		this.points = new Vector<Point2D>();

		// alle gesetzte dreiecke
		this.triangles = new Vector<Triangle>();

		// punkte, die ausgewhlt sind und kein vollstndiges dreieck bilden
		this.currentTriangle = new Vector<Point2D>();
		radius = 10;
	}

	private int[] scaledDistanceToRealDistance(int distanceX, int distanceY) {
		Point2D p = new Point2D.Double(distanceX, distanceY);
		p = this.inverseScaleTransform.transform(p, null);
		int pd[] = new int[2];
		pd[0] = (int) Math.round(p.getX());
		pd[1] = (int) Math.round(p.getY());
		return pd;
	}

	public ImagePanel(String filename) {
		this(new ImageIcon(filename).getImage());
	}

	public boolean addPoint(Point2D p) {
		Point2D p1 = new Point2D.Double(Math.round(p.getX()), Math.round(p
				.getY()));
		boolean added = (this.img != null) && !points.contains(p)
				&& !isInTriangle(p) && points.add(p1);
		if (added) {
			this.repaint();
			System.out.println("added P:" + p1);
		}
		return added;
	}

	private boolean isInTriangle(Point2D p) {
		boolean isIn = false;
		for (int i = 0; !isIn && (i < triangles.size()); i++) {
			isIn = triangles.get(i).isPointIn(p);
		}
		return isIn;
	}

	public Point2D removePoint(Point2D p) {
		int idx = points.indexOf(p);
		Point2D r_p = null;
		if (idx >= 0) {
			r_p = points.remove(idx);
		}
		return r_p;
	}

	public void moveToPoint(Point2D p, Point2D n_p) {
		int idx = points.indexOf(p);
		boolean edges = false;
		int i = 0;
		while (i < 4 && !edges) {
			edges = p == points.get(i++);
		}

		if ((idx >= 0) && !edges && !isPointSet(n_p)) {
			Point2D p_temp = (Point2D) p.clone();
			p.setLocation(n_p);
			boolean allow = true;
			for (int k = 0; allow && k < points.size(); k++) {
				Point2D p1 = points.get(k);
				for (int l = 0; allow && l < triangles.size(); l++) {
					Triangle t = triangles.get(l);
					System.out.println("is eine Ecke des Dreiecks " + t + " :"
							+ t.hasAConer(p1));
					allow = t.hasAConer(p1) || !t.isPointIn(p1);
				}
			}
			if (!allow) {
				System.out.println("darf nicht");
				p.setLocation(p_temp);
			}

			this.repaint();
			this.validate();
		}
	}

	public void movePoint(Point2D p, int x, int y) {
		int idx = points.indexOf(p);
		if (idx >= 0) {
			p.setLocation(p.getX() + x, p.getY() + y);
		}
	}

	public boolean isPointSet(Point2D p) {
		boolean found = false;
		int i = points.size() - 1;
		while (!found && (i >= 0)) {
			int[] r_real = this.scaledDistanceToRealDistance(radius, radius);
			found = (p.distance(points.get(i)) < Math.min(r_real[0], r_real[1]));
			if (!found)
				i--;
		}
		return found;
	}

	public boolean isImageLoaded() {
		return img != null;
	}

	private void setTransforms() {
		if (img != null) {
			int img_w = img.getWidth(null);
			int img_h = img.getHeight(null);
			double p_w = this.getWidth();
			double p_h = this.getHeight();
			// g2d.setClip(0, 0, img_w, img_h);

			scaleTransform = new AffineTransform(p_w / img_w, 0, 0,
					p_h / img_h, 0, 0);
			inverseScaleTransform = new AffineTransform(img_w / p_w, 0, 0,
					img_h / p_h, 0, 0);
		}
	}

	public void paint(Graphics g) {
		super.paint(g);
		Graphics2D g2d = (Graphics2D) g;
		// Shape s = g2d.getClip();
		if (img != null) {
			setTransforms();
			g2d.drawImage(this.img, scaleTransform, null);

			int l = points.size();
			for (int i = 0; i < l; i++) {
				Point2D p = points.get(i);
				Point2D p_transformed = scaleTransform.transform(p, null);
				int m = 0;
				for (int k = 0; k < triangles.size(); k++) {
					Triangle t = triangles.get(k);
					if (t.hasAConer(p))
						m |= 1;
				}
				for (int k = 0; k < currentTriangle.size(); k++) {
					if (currentTriangle.contains(p))
						m |= 2;
				}
				switch (m) {
				case 1:
					g2d.setColor(Color.GREEN);
					break;
				case 2:
				case 3:
					g2d.setColor(Color.YELLOW);
					break;
				default:
					g2d.setColor(Color.RED);
				}

				g2d.setStroke(new BasicStroke(5));
				g2d.drawOval((int) Math.round(p_transformed.getX() - radius),
						(int) Math.round(p_transformed.getY() - radius),
						radius * 2, radius * 2);
			}
			l = triangles.size();
			for (int i = 0; i < l; i++) {
				Triangle t = this.triangles.get(i);
				g2d.setColor(Color.BLUE);
				g2d.setStroke(new BasicStroke(3));
				int xs[] = new int[3];
				int ys[] = new int[3];
				Vector<Point2D> pnts = t.getConers();
				for (int n = 0; n < 3; n++) {
					Point2D p = pnts.get(n);
					p = scaleTransform.transform(p, null);
					xs[n] = (int) Math.round(p.getX());
					ys[n] = (int) Math.round(p.getY());
				}
				Polygon pol = new Polygon(xs, ys, 3);
				g2d.drawPolygon(pol);
				Color fill_c;
				if ((i != flashingTriangle) || (flashingTriangle == -1))
					fill_c = new Color(0, 0, 255, 70);
				else
					fill_c = new Color(255, 0, 0, 70);
				g2d.setColor(fill_c);
				g2d.setStroke(new BasicStroke(3));
				g2d.fill(pol);
			}
		}
		// g2d.setClip(s);
	}

	public void setImage(Image img, boolean left) {
		this.img = img;
		setTransforms();
		this.invalidate();
		if (left) {
			addPoint(new Point2D.Double(0, 0));
			addPoint(new Point2D.Double(0, img.getHeight(null)));
			addPoint(new Point2D.Double(img.getWidth(null), 0));
			addPoint(new Point2D.Double(img.getWidth(null), img.getHeight(null)));
		}
	}

	public Image getImage() {
		return this.img;
	}

	public Point2D getNearstPointTo(Point2D new_p) {
		boolean found = false;
		int i = 0;
		while (!found && (i < points.size())) {
			int[] r_real = this.scaledDistanceToRealDistance(radius, radius);
			found = (new_p.distance(points.get(i)) < Math.min(r_real[0],
					r_real[1]));
			if (!found)
				i++;
		}
		if (found)
			return points.get(i);
		else
			return null;
	}

	public AffineTransform getScaleTransform() {
		return this.scaleTransform;
	}

	public AffineTransform getInverseScaleTransform() {
		return this.inverseScaleTransform;
	}

	public boolean addTriangle(Vector<Point2D> trianglePoints) {
		boolean added = false;
		Triangle t = new Triangle(trianglePoints.get(0), trianglePoints.get(1),
				trianglePoints.get(2));

		// ein punkt darf nicht im dreieck enthalten sein
		int i = 0;
		while ((i < points.size())
				&& (((t.hasAConer(points.get(i)))) || (!t.isPointIn(points
						.get(i)))))
			i++;

		// dasselbe dreieck darf nicht ausgewhlt werden
		if ((i == points.size()) && (!triangles.contains(t))) {
			triangles.addElement(t);
			System.out.println("added");
			added = true;
			this.currentTriangle.clear();
		}
		if (added)
			this.repaint();
		return added;
	}

	public Vector<Line2D> getAllEdges() {
		Vector<Line2D> v = new Vector<Line2D>();
		for (int i = 0; i < triangles.size(); i++) {
			v.addAll(triangles.get(i).getEdges());
		}
		return v;
	}

	public boolean intersectsAnEdge(Point2D old_p, Point2D new_p,
			Vector<Line2D> allEdges) {
		Vector<Point2D> ps = new Vector<Point2D>();
		for (int i = 0; i < triangles.size(); i++) {
			Triangle t = triangles.get(i);
			if (t.hasAConer(old_p)) {
				Vector<Point2D> cs = t.getConers();
				for (int j = 0; j < cs.size(); j++) {
					if (old_p != cs.get(j) && (!ps.contains(cs.get(j))))
						ps.add(cs.get(j));
				}
			}
		}

		Vector<Line2D> n_allEdges = new Vector<Line2D>();
		for (int i = 0; i < allEdges.size(); i++) {
			Line2D l = allEdges.get(i);
			if (l.getP1().equals(old_p)) {
				n_allEdges.add(new Line2D.Double(new_p, l.getP2()));
			} else if (l.getP2().equals(old_p)) {
				n_allEdges.add(new Line2D.Double(new_p, l.getP1()));
			} else {
				n_allEdges.add(l);
			}
		}

		boolean intersects = false;
		for (int i = 0; !intersects && i < ps.size(); i++) {
			Line2D l = new Line2D.Double(new_p, ps.get(i));
			intersects = intersectsAnEdge(l, n_allEdges);
		}
		return intersects;
	}

	public boolean intersectsAnEdge(Line2D l, Vector<Line2D> allEdges) {
		boolean intersects = false;
		for (int j = 0; (j < allEdges.size()) && (!intersects); j++) {
			Vector<Point2D> punktMenge = new Vector<Point2D>();
			punktMenge.add(allEdges.get(j).getP1());
			punktMenge.add(allEdges.get(j).getP2());
			intersects = l.intersectsLine(allEdges.get(j))
					&& !punktMenge.contains(l.getP1())
					&& !punktMenge.contains(l.getP2());
		}

		return intersects;
	}

	public boolean addTriangleConer(Point2D p) {
		Vector<Line2D> allEdges = getAllEdges();
		currentTriangle.addElement(p);
		Vector<Line2D> cE = getCurrentEdges();
		boolean intersects = false;
		for (int i = 0; i < cE.size() && (!intersects); i++) {
			intersects = intersectsAnEdge(cE.get(i), allEdges);
			/*
			 * for (int j = 0; (j < allEdges.size()) && (!intersects); j++) {
			 * Vector<Point2D> punktMenge = new Vector<Point2D>();
			 * punktMenge.add(allEdges.get(j).getP1());
			 * punktMenge.add(allEdges.get(j).getP2()); intersects =
			 * cE.get(i).intersectsLine(allEdges.get(j)) &&
			 * !punktMenge.contains(cE.get(i).getP1()) &&
			 * !punktMenge.contains(cE.get(i).getP2()); }
			 */
		}

		if (intersects) {
			currentTriangle.removeElement(p);
		} else {
			this.repaint();
		}
		return !intersects;
	}

	private Vector<Line2D> getCurrentEdges() {
		Vector<Line2D> edges = new Vector<Line2D>();
		Line2D l;
		switch (currentTriangle.size()) {
		case 2:
			l = new Line2D.Double(currentTriangle.get(0), currentTriangle
					.get(1));
			edges.addElement(l);
			break;
		case 3:
			l = new Line2D.Double(currentTriangle.get(1), currentTriangle
					.get(2));
			edges.addElement(l);
			l = new Line2D.Double(currentTriangle.get(2), currentTriangle
					.get(0));
			edges.addElement(l);
		default:
			;
		}
		return edges;
	}

	public void removeTriangleConer(Point2D p) {
		currentTriangle.removeElement(p);
		this.repaint();
		System.out.println("size:" + currentTriangle.size());
	}

	public boolean completeToCommit() {
		boolean complete = img != null;
		for (int x = 0; complete && (x < img.getWidth(null)); x++) {
			for (int y = 0; complete && (y < img.getHeight(null)); y++) {
				boolean belongs = false;
				for (int i = 0; !belongs && (i < triangles.size()); i++) {
					belongs = triangles.get(i).isPointIn(
							new Point2D.Double(x, y));
				}
				// System.out.println("belongs="+belongs);
				complete = belongs;
				if (!belongs)
					System.out.println(new Point2D.Double(x, y));
			}
		}
		return complete;
	}

	public Vector<Triangle> getTriangles() {
		Vector<Triangle> triangles = new Vector<Triangle>();
		for (int i = 0; i < this.triangles.size(); i++) {
			triangles.addElement((Triangle) this.triangles.get(i).clone());
		}
		return triangles;
	}

	public Vector<Point2D> getPoints() {
		Vector<Point2D> ps = new Vector<Point2D>();
		for (int i = 0; i < points.size(); i++) {
			ps.addElement(this.points.get(i));
		}
		return ps;
	}

	public void setPoints(Vector<Point2D> points) {
		this.points = new Vector<Point2D>();
		this.triangles = new Vector<Triangle>();
		for (int i = 0; i < points.size(); i++) {
			addPoint(points.get(i));
		}
	}

	public void setTriangles(Vector<Point2D> points, Vector<Triangle> triangles) {
		setPoints(points);
		for (int i = 0; i < triangles.size(); i++) {
			Triangle t = triangles.get(i);
			Vector<Point2D> ps = t.getConers();
			Vector<Point2D> ps1 = new Vector<Point2D>();
			for (int j = 0; j < 3; j++) {
				ps1.addElement(getNearstPointTo(ps.get(j)));
			}
			addTriangle(ps1);
		}
		scaleToImageSize();

	}

	private void scaleToImageSize() {
		double mx_x = -1;
		double mx_y = -1;
		for (int i = 0; i < points.size(); i++) {
			Point2D p = points.get(i);
			mx_x = mx_x < p.getX() ? p.getX() : mx_x;
			mx_y = mx_y < p.getY() ? p.getY() : mx_y;
		}
		double f_x = img.getWidth(null) / mx_x;
		double f_y = img.getHeight(null) / mx_y;
		for (int i = 0; i < points.size(); i++) {
			Point2D p = points.get(i);
			p.setLocation(Math.round(p.getX() * f_x), Math
					.round(p.getY() * f_y));
		}
	}

	public void flashTriangle(int i, boolean b) {
		if (b)
			flashingTriangle = i;
		else
			flashingTriangle = -1;

	}

	public void clearAll() {
		this.points.clear();
		this.triangles.clear();
		
	}

}
